

################################################################################


class Derivative:
  # represents the derivative of a magnetization
  
  
#-----------------------------------------------------------------------------#


  def __init__(self, mesh, V, X, maxwellCoupling, solvetol):
    from ngsolve import GridFunction
    from . import strayField, oerstedField, _zeroField
    # X is the compound FESpace \subset H1(\Omega)^3
    self._mesh = mesh
    
    self.maxwellCoupling = maxwellCoupling
    
    self.gf = GridFunction(X)
    self.Dgf = self.gf.Deriv()
    self.cf = self.gf
    
    if( self.maxwellCoupling == "magnetostaticField" \
      or self.maxwellCoupling == "strayField"):
      self.hs = strayField.StrayField(self._mesh, self.cf, solvetol)
  

#-----------------------------------------------------------------------------#


  def UpdateMaxwellCoupling(self, newMaxwellCoupling, solvetol):
    from . import strayField

    self.maxwellCoupling = newMaxwellCoupling
    self.hs = None

    if( self.maxwellCoupling == "magnetostaticField" \
      or self.maxwellCoupling == "strayField"):
      self.hs = strayField.StrayField(self._mesh, self.cf, solvetol)

    
#------------------------------------------------------------------------------#


  def ComputeMaxwell(self, solvetol):
    import sys

    if( self.maxwellCoupling == "magnetostaticField" \
      or self.maxwellCoupling == "strayField"):
      self._ComputeStrayField(solvetol)
    elif(self.maxwellCoupling != "none"):
      print("Error in _Mag.ComputeMaxwell:")
      print("maxwellCoupling =", self.maxwellCoupling, \
        "unknown to Magnetization.")
      sys.exit()
  

#------------------------------------------------------------------------------#

  
  def _ComputeStrayField(self, solvetol):
    import sys
    if ( self.maxwellCoupling != "strayField" \
      and self.maxwellCoupling != "magnetostaticField" ):
      print("Error in _Mag.v._ComputeStrayField():")
      print("maxwellCoupling != <strayField> !! --> Terminate.")
      sys.exit()
    
    self.hs.Compute(solvetol)
  
  
#-----------------------------------------------------------------------------#


  def phiUniaxialAnisotropy(self, e):
    from commics._tools.algebra import InnerProd
    return -1.0 * InnerProd(e, self.gf) * e
  
  
#-----------------------------------------------------------------------------#


  def opZhangLi(self, vel):
    return (sum(vel[k]*self.Dgf[0,k] for k in range(3)), \
      sum(vel[k]*self.Dgf[1,k] for k in range(3)), \
      sum(vel[k]*self.Dgf[2,k] for k in range(3)) )


#-----------------------------------------------------------------------------#


################################################################################
