### class OutputOptions ###


################################################################################


class OutputOptions:
  '''
  Objects of this class are meant to control what is written out during simulation
  However, we suggest using the .SetXYZ() methods from the Integrator class,
  those are available for the most important members of this class.
  --> See help(Integrator)
  '''
  # description 
  
  def __init__(self, other=None):
    if( other == None ):
      self.foldername = "result"
      
      self.filenameParameters = "parameters.txt"
      self.filenameAverages = "averages.dat"
      self.filenameExtField = "appliedFieldAtPoint.dat"
      self.filenameCurrent = "current.dat"
      self.filenameNorms = "norms.dat"
      self.filenamePointwiseAverages = "pointwise_averages.dat"
      self.filenameCompTimes = "computationalTime.dat"
      self.filenameEnergies = "energies.dat"
      self.filenameUnprojectedExchangeEnergy = "unprojectedExchangeEnergy.dat"
      self.filenamePlotEnergies = "plotEnergies.eps"
      self.filenameIterationCount = "iterations.dat"
      self.filenameAdaptiveAxis = "adaptiveAxis.dat"
      
      self.columnWidth = 25
      
      self.headerParameters = "This file contains the simulation parameters."
      self.headerAverages = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "<m_x>,".ljust(self.columnWidth) + " " \
        + "<m_y>,".ljust(self.columnWidth) + " " \
        + "<m_z>,".ljust(self.columnWidth)
      self.headerExtField = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "H_x,".ljust(self.columnWidth) + " " \
        + "H_y,".ljust(self.columnWidth) + " " \
        + "H_z,".ljust(self.columnWidth)
      self.headerCurrent = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "Je(sl),".ljust(self.columnWidth) + " " \
        + "Je(zl)_x,".ljust(self.columnWidth) + " " \
        + "Je(zl)_y,".ljust(self.columnWidth) + " " \
        + "Je(zl)_z,".ljust(self.columnWidth)
      self.headerNorms = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "Linfty-Norm,".ljust(self.columnWidth) + " " \
        + "Int(I_h|m_h|^2-1),".ljust(self.columnWidth)
      self.headerPointwiseAverages = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "<m_x>,".ljust(self.columnWidth) + " " \
        + "<m_y>,".ljust(self.columnWidth) + " " \
        + "<m_z>,".ljust(self.columnWidth)
      
      self.headerCompTimes = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "this step [s],".ljust(self.columnWidth) + " " \
        + "cumulative [s],".ljust(self.columnWidth)
      
      self.headerEnergies = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "energy_exchange,".ljust(self.columnWidth) + " " \
        + "energy_anisotropy,".ljust(self.columnWidth) + " " \
        + "energy_dm,".ljust(self.columnWidth) + " " \
        + "energy_strayField,".ljust(self.columnWidth) + " " \
        + "energy_applied,".ljust(self.columnWidth) + " " \
        + "energy_total,".ljust(self.columnWidth)
      
      self.headerUnprojectedExchangeEnergy \
        = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "unprojected_energy_exchange,".ljust(self.columnWidth)

      self.headerIterationCount = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "gmres iter count,".ljust(self.columnWidth)

      self.headerAdaptiveAxis = "columns are: t,".ljust(self.columnWidth-2) + " " \
        + "adaptiveAxis,".ljust(self.columnWidth) + " " \
        + "adaptiveGamma,".ljust(self.columnWidth) + " " \
        + "d1minus,".ljust(self.columnWidth) + " " \
        + "d1plus,".ljust(self.columnWidth) + " " \
        + "d2minus,".ljust(self.columnWidth) + " " \
        + "d2plus,".ljust(self.columnWidth) + " " \
        + "d3minus,".ljust(self.columnWidth) + " " \
        + "d3plus,".ljust(self.columnWidth)

      self.foldernameVideo = "mag_video"
      self.filenameMagnetization = "mag"
      self.filenameStrayField = "stray"
      self.filenameOerstedField = "oersted"
      self.filenameVideotimes = "timeseries.dat"
      self.filenameMesh = "mesh.vol"
      
      self.draw = True
      self.video = True
      self.videoMaxwell = False
      self.saveatevery = 200
      self.memoryWarning = False

      self.traceUnprojectedExchangeEnergy = False

    else:
      for key, value in other.__dict__.items():
        self.__setattr__(key, value)


#------------------------------------------------------------------------------#


  def __str__(self):
    # this doesn't need a print option
    return ""
#    string = "\n\n ----- " + self.__class__.__name__ + " ----- \n\n"
#    for key, value in self.__dict__.items():
#      if not(key.startswith("_")):
#        string += key + " = " + str(value) + "\n"
#    
#    return string


#------------------------------------------------------------------------------#


  def DeepCopy(self): 

    copy = type(self)()
    for key, value in self.__dict__.items():
      copy.__setattr__(key, value)

    return copy


#------------------------------------------------------------------------------#


################################################################################
