from commics import *

# specify geometry and parameters
geo = Geometry(geometry=Cuboid(120, 120, 10), meshSize=(2, 2, 2), \
               structuredMesh=True, scaling=1.0e-09)
par = Parameters(A=1.3e-11, Ms=8.0e+05, K=0.0, gamma0=2.210173e+05, \
                 alpha=1.0, timeStepSize=0.1e-12, T_start=-5.0e-9, \
                 m0=(0, 0, 1))

# define integrator, choose number of threads, specify folder for results
fmr = Integrator(scheme=TPS2AB, geometry=geo, parameters=par, numthreads=4)
fmr.SetResultsFolder("FMR_Result")

# obtain initial condition
amplitude = 80.0e+03
e = (1.0, 0.715, 0.0)
e_length = (sum (e[j]**2 for j in range(3)))**0.5
H_ext = (amplitude*e[0]/e_length, amplitude*e[1]/e_length, 0.0)
fmr.SetParameter_H_ext(H_ext)
fmr.Integrate(duration=5.0e-09)

# change direction of applied field and run simulation
fmr.SetParameter_alpha(0.008)
e = (1.0, 0.7, 0.0)
e_length = (sum(e[j]**2 for j in range(3)))**0.5
H_ext = (amplitude*e[0]/e_length, amplitude*e[1]/e_length, 0.0)
fmr.SetParameter_H_ext(H_ext)
fmr.Integrate(duration=20.0e-09)
