from commics import *

# specify geometry and parameters
geo = Geometry(geometry=Disk(120.0, 10.0), meshSize=7.5, \
               forceNetgenMeshSize=True, scaling=1.0e-09)
par = Parameters(A=8.78e-12, Ms=3.84e+05, D=1.58e-03, dmCoupling="bulk", \
                 spintronicsCoupling="slonczewski", d=10.0e-09, P=0.4, \
                 p =(0.0, 1.0, 0.0), \
                 alpha=1.0, timeStepSize=1e-13, theta=1.0, m0=(0, 0, 1))

# define integrator, choose number of threads
helimag = Integrator(scheme=TPS1, geometry=geo, parameters=par, numthreads=4)

# choose to save magnetization as . vtk - file every X seconds, run simulation
helimag.RecordMagnetization(every=10.0e-12)
helimag.Integrate(duration=2.0e-9, relax=True)

# prepare JeMax, alpha, and points in time
helimag.SetParameter_alpha(0.002)
JeMax = 1.0e+12
T0, T1, T2, T3 = 0, 40.0e-12, 110.0e-12, 150.0e-12

# set increasing current, run simulation
helimag.SetParameter_Je(lambda t, x, y, z: (t-T0) /(T1-T0)*JeMax)
helimag.Integrate(duration=T1-T0)

# set constant current, run simulation
helimag.SetParameter_Je(JeMax)
helimag.Integrate(duration=T2-T1)

# set decreasing current, run simulation
helimag.SetParameter_Je(lambda t, x, y, z: (T3-t)/(T3-T2)*JeMax)
helimag.Integrate(duration=T3-T2)

# final relaxation
helimag.Integrate(duration=20.0e-9, relax=True)
