from commics import *

# specify geometry and parameters
h_xy , h_z = 125/69, 1.5
geo = Geometry(geometry=Cuboid (500, 125, 3), meshSize=(h_xy, h_xy, h_z), \
               structuredMesh=True, scaling=1e-9)
par = Parameters(A=1.3e-11, Ms=8e+5, K=0, D=0, gamma0=2.211e+5, \
                  alpha=1.0, m0=(1.0, 0.0, 0.0), \
                  T_start=0, T_end=1e-9, timeStepSize=0.1e-12)

# define time dependent applied field via lambda function
from math import sqrt
field = lambda t, x, y, z: (par.T_end - t) / (par.T_end - par.T_start) \
                           * 30.0e-3 / par.mu0 / sqrt(3.0)
par.H_ext = (field, field, field)

# define integrator and run simulation from T_start to T_end
sp4 = Integrator(scheme=TPS2AB, geometry=geo, parameters=par, numthreads=4)
sp4.Integrate()

# Relax for another nanosecond and save the mesh and the result
sp4.Integrate(duration=1e-9, relax=True)
sp4.SaveMesh("sp4mesh")
sp4.SaveMagnetization("sp4sState")
