from commics import *

# specify geometry and parameters
h = 5 / 3
geo = Geometry(geometry=Cuboid((-50, -50, -5), (50,50 ,5)), meshSize=(h, h, h), \
               structuredMesh=True, scaling=1.0e-09)

par = Parameters(A=1.3e-11, Ms=8.0e+05, K=0.0, gamma0=2.211e+05, alpha=1.0, \
                 spintronicsCoupling="zhangLi", g=2.0, P=1.0, \
                 Je=(1.0e+12, 0.0, 0.0), xi=0.05, \
                 timeStepSize=0.1e-12, T_start=-1.0e-09)

# space dependent initial condition ( scaled domain ); normalized automatically
from ngsolve import x, y
par.m0 = (-y, x, 10.0)

# define integrator and relax configuration
sp5 = Integrator(scheme=TPS2AB, geometry=geo, parameters=par, numthreads=4)
sp5.Integrate(duration=1.0e-09, relax=True)

# set damping alpha and run simulation with specified current
sp5.SetParameter_alpha(0.1)
sp5.Integrate(duration=8.0e-09)
