from . import _interface

################################################################################


class WriteInterface(_interface._Interface):
  

#------------------------------------------------------------------------------#


  def _CreateOutputFolder(self):
    from commics._tools.output import CreateNewFolder
    if (self._newFolder):
      output_dir = CreateNewFolder(self.output.foldername)
      print("Folder --", output_dir, "-- for saving results created.")
      self._newFolder = False
      self.output.foldername = output_dir
      
      if self.output.memoryWarning:
        videoMemoryWarning(mesh = self._geometry.GetMesh() \
          , simulationTime = self.parameters.T_end - self.parameters.T_start \
          , k_ = self.parameters.timeStepSize \
          , saveatevery = self.output.saveatevery)
      video_dir = CreateNewFolder(name = self.output.foldername + "/" \
        + self.output.foldernameVideo)
      print("Folder --", video_dir, "-- for saving video created.")
      self.output.foldernameVideo = video_dir
    else:
      print("Existing folder --", self.output.foldername \
        , "-- will be used for saving results.")


#------------------------------------------------------------------------------#


  def SaveMagnetization(self, filename=None):
    from commics._tools.output import LiveSaveValues
    
    if( filename == None):
      self._Mag.Save(self.output.foldernameVideo + "/" \
          + self.output.filenameMagnetization + "_" + str(self._counter))
    
      if (self._counter == 0):
        timeHeader = "colums are: t,".ljust(self.output.columnWidth) \
          + "step_nr".ljust(self.output.columnWidth)
      else:
        timeHeader = None
          
      LiveSaveValues(self.output.foldernameVideo + "/" \
        + self.output.filenameVideotimes, self.Time() \
        , [self._counter], nrwidth = self.output.columnWidth \
        , header = timeHeader)
    else:
      self._Mag.Save(self.output.foldername + "/" + filename)
  
  
#------------------------------------------------------------------------------#


  def SaveMaxwell(self):
    
    if(self.parameters.maxwellCoupling == "strayField"):
      self._Mag.SaveStrayField(self.output.foldernameVideo + "/" \
          + self.output.filenameStrayField + "_" + str(self._counter))
    elif(self.parameters.maxwellCoupling == "oerstedField"):
      self._hc.Save(self.output.foldernameVideo + "/" \
          + self.output.filenameOerstedField + "_" + str(self._counter))
    elif(self.parameters.maxwellCoupling == "magnetostaticField"):
      self._Mag.SaveStrayField(self.output.foldernameVideo + "/" \
          + self.output.filenameStrayField + "_" + str(self._counter))
      self._hc.Save(self.output.foldernameVideo + "/" \
          + self.output.filenameOerstedField + "_" + str(self._counter))
    else:
      pass
  
  
#------------------------------------------------------------------------------#


  def SaveMesh(self, filename=None):
    
    if( filename == None):
      filename = self.output.filenameMesh
    
    self._geometry.Save( self.output.foldername + "/" + filename )
  
  
#------------------------------------------------------------------------------#


  def Write(self):
  
    # TODO make these conditional on counter, i.e., not write at every timestep
    self._WriteAverages()
    self._WritePointwiseAverages()
    self._WriteExternalField() 
    self._WriteCurrent() 
    self._WriteNorms()
    self._WriteComputationalTime()
    self._WriteEnergies()
    self._WriteUnprojectedExchangeEnergy()
    self._WriteMagnetization()
    self._WriteMaxwell()
    self._WriteIterations()
    self._WriteAdaptiveAxis()


#------------------------------------------------------------------------------#


  def _WriteAverages(self):
    from commics._tools.output import LiveSaveValues
    ## averages
    headerAvg = self.output.headerAverages if self._counter == 0 else None
    avg = self._Mag.Averages()
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameAverages \
      , self.Time(), avg, nrwidth=self.output.columnWidth, header=headerAvg)
    

#------------------------------------------------------------------------------#


  def _WritePointwiseAverages(self):
    from commics._tools.output import LiveSaveValues
    ## pointwise averages
    headerPointwiseAvg \
      = self.output.headerPointwiseAverages if self._counter == 0 else None
    pw_avg = self._Mag.PointwiseAverages()
    LiveSaveValues(self.output.foldername + "/" \
      + self.output.filenamePointwiseAverages \
      , self.Time(), pw_avg, nrwidth=self.output.columnWidth \
      , header=headerPointwiseAvg)
    

#------------------------------------------------------------------------------#


  def _WriteExternalField(self):
    from commics._tools.output import LiveSaveValues
    ## external field
    headerExt = self.output.headerExtField if self._counter == 0 else None
    ext = self._EvaluateExternalFieldAtPoint()
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameExtField \
      , self.Time(), ext, nrwidth=self.output.columnWidth, header=headerExt)
    

#------------------------------------------------------------------------------#


  def _WriteCurrent(self):
    from commics._tools.output import LiveSaveValues
    ## current
    headerCur = self.output.headerCurrent if self._counter == 0 else None
    cur = self._EvaluateCurrent()
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameCurrent \
      , self.Time(), cur, nrwidth=self.output.columnWidth, header=headerCur)
    

#------------------------------------------------------------------------------#


  def _WriteNorms(self):
    from commics._tools.output import LiveSaveValues
    ## norms
    headerNrm = self.output.headerNorms if self._counter == 0 else None
    nrm = self._Mag.Norms()
    nrm[1] *= self._geometry.scaling \
      * self._geometry.scaling*self._geometry.scaling
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameNorms \
      , self.Time(), nrm, nrwidth=self.output.columnWidth, header=headerNrm)
    

#------------------------------------------------------------------------------#


  def _WriteComputationalTime(self):
    from commics._tools.output import LiveSaveValues
    ## computational Times
    headerCompTimes = self.output.headerCompTimes if self._counter == 0 else None
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameCompTimes \
      , self.Time(), [self._compTimeNow, self._compTimeTotal] \
      , nrwidth=self.output.columnWidth \
      , header=headerCompTimes)
    

#------------------------------------------------------------------------------#


  def _WriteEnergies(self):
    from commics._tools.output import LiveSaveValues

    ## energies
    headerEnergy = self.output.headerEnergies if self._counter == 0 else None
    
    E_exch = self.parameters.A * self._geometry.scaling \
      * self._Mag.EnergyExchange()
    
    if (self.parameters.K == 0):
      E_ani = 0.0
    else:
      E_ani = self._geometry.scaling \
        * self._geometry.scaling * self._geometry.scaling \
        * self.parameters.K \
        * self._Mag.EnergyUniaxialAnisotropy(self._easy_cf)
    
    if ( (self.parameters.D == 0) or (self.parameters.dmCoupling == "none") ):
      E_dm = 0.0
    elif( self.parameters.dmCoupling == "bulk" ):
      E_dm = self._geometry.scaling * self._geometry.scaling \
        * self.parameters.D \
        * self._Mag.EnergyBulkDzyaloshinskiiMoriya()
    elif( self.parameters.dmCoupling == "interfacial" ):
      E_dm = self._geometry.scaling * self._geometry.scaling \
        * self.parameters.D \
        * self._Mag.EnergyInterfacialDzyaloshinskiiMoriya()
    
    if (self.parameters.maxwellCoupling == "strayField" \
      or self.parameters.maxwellCoupling == "magnetostaticField" ):
      E_stray = -0.5 * self.parameters.mu0 \
        * self.parameters.Ms * self.parameters.Ms \
        * self._geometry.scaling * self._geometry.scaling \
        * self._geometry.scaling * self._Mag.EnergyStrayField()
    else:
      E_stray = 0.0
    
    E_appl = -1.0 * self.parameters.mu0 \
      * self.parameters.Ms * self.parameters.Ms \
      * self._geometry.scaling * self._geometry.scaling \
      * self._geometry.scaling * self._Mag.EnergyApplied(self._hext_cf)
      
    E_total = E_exch + E_ani + E_dm + E_stray + E_appl
    
    en = [E_exch, E_ani, E_dm, E_stray, E_appl, E_total]
    
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameEnergies \
      , self.Time(), en, nrwidth=self.output.columnWidth \
      , header=headerEnergy)
  

#------------------------------------------------------------------------------#


  def _WriteUnprojectedExchangeEnergy(self):
    if not(self.output.traceUnprojectedExchangeEnergy): return

    from commics._tools.output import LiveSaveValues
    header = self.output.headerUnprojectedExchangeEnergy if self._counter == 0 else None
    E_exch = self.parameters.A * self._geometry.scaling \
      * self._MagUnprojected.EnergyExchange()
    
    LiveSaveValues(self.output.foldername+"/" \
      + self.output.filenameUnprojectedExchangeEnergy \
      , self.Time(), [E_exch], nrwidth=self.output.columnWidth \
      , header=header)


#------------------------------------------------------------------------------#


  def _WriteMagnetization(self):
    if self._counter == 0:
      self.SaveMagnetization(self.output.filenameMagnetization + "_" + "initial")
      self.SaveMesh()
    
    if self.output.video == False: return

    if ((self._counter - self._startCounter) % self.output.saveatevery == 0):
      self.SaveMagnetization()
    
  
#------------------------------------------------------------------------------#


  def _WriteMaxwell(self):
    if self.output.videoMaxwell == False: return

    if self._counter == 0:
      self._ComputeMaxwell()
      self.SaveMaxwell()
    
    if ((self._counter - self._startCounter) % self.output.saveatevery == 0):
      self.SaveMaxwell()
    
  
#------------------------------------------------------------------------------#


  def _WriteIterations(self):
    from commics._tools.output import LiveSaveValues
    from commics.integrators._details import IterationCounter
    if not isinstance(self._callback, IterationCounter): return

    ## iterations
    headerItc = self.output.headerIterationCount if self._counter == 0 else None
    itc = [self._callback.GetCount()]
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameIterationCount \
      , self.Time(), itc, nrwidth=self.output.columnWidth, header=headerItc)
    

#------------------------------------------------------------------------------#


  def _WriteAdaptiveAxis(self):
    from commics._tools.output import LiveSaveValues
    if not(self.parameters.QmatAxis == "adaptive"): return

    ## axis
    headerAax = self.output.headerAdaptiveAxis if self._counter == 0 else None
    aax = [self._adaptiveAxis, self._adaptiveGamma] + self._adaptiveAxis_d_vals
    LiveSaveValues(self.output.foldername+"/"+self.output.filenameAdaptiveAxis \
      , self.Time(), aax, nrwidth=self.output.columnWidth, header=headerAax)
    

#------------------------------------------------------------------------------#


  def SaveParameters(self):
    from commics._tools.output import SaveContent
    # TODO: sort the output stuff, probably brute force?
    # TODO: I want output: "This is the 1./2./3. simualtion stage...
    saveString = ""
    if (self._counter == 0):
      saveString += self.output.headerParameters
      saveString += "\n" + "\n"
    
    saveString += str(self)
    saveString += "\n" + "\n"
    saveString += "########################################"
    saveString += "########################################"
    saveString += "\n" + "\n"
    
    SaveContent(self.output.foldername + "/" \
      + self.output.filenameParameters, saveString)
  
  
#------------------------------------------------------------------------------#


################################################################################
