

################################################################################


#------------------------------------------------------------------------------#


def PhysicalToMathematical(self):
  ConvertTime(self)
  ComputeConstants(self)
  ConvertAppliedField(self)
  ConvertEasyAxis(self)
  ConvertCurrentDensity(self)
  ConvertPolarizationAxis(self)


#------------------------------------------------------------------------------#


def ConvertTime(self):
  from ngsolve import Parameter
  self._k = self.parameters.gamma0 * self.parameters.Ms \
    * self.parameters.timeStepSize
  self._t = self.parameters.gamma0 * self.parameters.Ms \
    * self.parameters.T_start
  self._time_cf = Parameter( self._t )
  self._T_end = self.parameters.gamma0 * self.parameters.Ms \
    * self.parameters.T_end

  self._steps = round((self.parameters.T_end - self.parameters.T_start) \
    / self.parameters.timeStepSize)


#------------------------------------------------------------------------------#


def ComputeConstants(self):
  Compute_Cex(self)
  Compute_Cani(self)
  Compute_Cdm(self)
  Compute_Cst(self)
  

#------------------------------------------------------------------------------#


def Compute_Cex(self):
  self._C_ex = 2.0*self.parameters.A \
    / ( self.parameters.mu0 \
    * self.parameters.Ms * self.parameters.Ms \
    * self._geometry.scaling * self._geometry.scaling )
  

#------------------------------------------------------------------------------#


def Compute_Cani(self):
  self._C_ani = 2.0*self.parameters.K / ( self.parameters.mu0 \
    * self.parameters.Ms * self.parameters.Ms )
  

#------------------------------------------------------------------------------#
  
  
def Compute_Cdm(self):
  import sys
  if( (self.parameters.dmCoupling == "bulk") \
    or (self.parameters.dmCoupling == "interfacial") ):
    self._C_dm = 2.0*self.parameters.D / ( self.parameters.mu0 \
      * self.parameters.Ms * self.parameters.Ms \
      * self._geometry.scaling )
  elif( self.parameters.dmCoupling == "none" ):
    self._C_dm = 0.0
  else:
    print("Error in _PhysicalToMathematical():")
    print("Dzyaloshinskii-Moriya options not >bulk<, >interfacial< \
      or >none< are not implemented yet")
    sys.exit()


#------------------------------------------------------------------------------#
  

def Compute_Cst(self):
  import sys
  if (self.parameters.spintronicsCoupling == "zhangLi"):
    self._C_st = self.parameters.P * self.parameters.g \
      * self.parameters.muB / 2.0 / self.parameters.gamma0 \
      / self.parameters.Ms / self._geometry.scaling \
      / self.parameters.e
  elif (self.parameters.spintronicsCoupling == "slonczewski"):
    self._C_st = self.parameters.hbar \
      / self.parameters.e / self.parameters.mu0 \
      / self.parameters.Ms / self.parameters.d
  elif (self.parameters.spintronicsCoupling == "none"):
    self._C_st = 0.0
  else:
    print("Error in _PhysicalToMathematical():")
    print("Spintronics options not >zhangLi<, >slonczewski< \
      or >none< are not implemented yet")
    sys.exit()
  
  
#------------------------------------------------------------------------------#
  
  
def ConvertAppliedField(self):
  import types
  from ngsolve import CoefficientFunction, x, y, z
  if(self.parameters.H_ext == None):
    return
  
  tScale = self.parameters.gamma0 * self.parameters.Ms

  hext = [0.0, 0.0, 0.0]
  hext_impl = [0.0, 0.0, 0.0]
  hext_mp = [0.0, 0.0, 0.0]
  for d in range(3):
    if isinstance(self.parameters.H_ext[d], types.FunctionType):
      hext[d] = self.parameters.H_ext[d](self._time_cf / tScale, x, y, z)
      hext_impl[d] = self.parameters.H_ext[d]( \
        (self._time_cf + self._theta*self._k) / tScale, x, y, z)
      hext_mp[d] = self.parameters.H_ext[d]( \
        (self._time_cf + 0.5*self._k) / tScale, x, y, z)
    else:
      hext[d] = CoefficientFunction(self.parameters.H_ext[d])
      hext_impl[d] = CoefficientFunction(self.parameters.H_ext[d])
      hext_mp[d] = CoefficientFunction(self.parameters.H_ext[d])

  self._hext_cf = CoefficientFunction(tuple(hext)) / self.parameters.Ms
  self._hext_impl_cf = CoefficientFunction(tuple(hext_impl)) / self.parameters.Ms
  self._hext_mp_cf = CoefficientFunction(tuple(hext_mp)) / self.parameters.Ms


#------------------------------------------------------------------------------#


def ConvertEasyAxis(self):
  import scipy
  from ngsolve import CoefficientFunction
  if(self.parameters.K == 0.0):
    return

  easy_norm = scipy.sqrt( sum(self.parameters.easy_axis[i] \
    * self.parameters.easy_axis[i] for i in range(3)) )
  
  self._easy_cf = CoefficientFunction(self.parameters.easy_axis) / easy_norm


#------------------------------------------------------------------------------#


def ConvertCurrentDensity(self):
  import sys
  if (self.parameters.spintronicsCoupling == "zhangLi"):
    ConvertCurrentDensity_ZhangLi(self)
  elif (self.parameters.spintronicsCoupling == "slonczewski"):
    ConvertCurrentDensity_Slonczewski(self)
  elif (self.parameters.spintronicsCoupling == "none"):
    self._j_cf = None
  else:
    print("Error in _PhysicalToMathematical():")
    print("Spintronics options not >zhangLi<, >slonczewski< \
      or >none< are not implemented yet")
    sys.exit()


#------------------------------------------------------------------------------#


def ConvertCurrentDensity_ZhangLi(self):
  import sys
  import types
  from ngsolve import CoefficientFunction, x, y, z
  if len(self.parameters.Je) != 3:
    print("Error in _PhysicalToMathematical():")
    print("len(Je) must be exactly 3 for spintronicsCoupling=zhangLi")
    sys.exit()

  flag_je_const = True

  j = [0.0, 0.0, 0.0]
  j_impl = [0.0, 0.0, 0.0]
  j_mp = [0.0, 0.0, 0.0]
  tScale = self.parameters.gamma0 * self.parameters.Ms
  for d in range(3):
    if isinstance(self.parameters.Je[d], types.FunctionType):
      flag_je_const = False
      j[d] = self.parameters.Je[d](self._time_cf / tScale, x, y, z)
      j_impl[d] = self.parameters.Je[d]( \
        (self._time_cf + self._theta*self._k) / tScale, x, y, z)
      j_mp[d] = self.parameters.Je[d]( \
        (self._time_cf + 0.5*self._k) / tScale, x, y, z)
    else:
      j[d] = CoefficientFunction(self.parameters.Je[d])
      j_impl[d] = CoefficientFunction(self.parameters.Je[d])
      j_mp[d] = CoefficientFunction(self.parameters.Je[d])

  self._j_cf = CoefficientFunction(tuple(j)) / self.parameters.Ms
  self._j_impl_cf = CoefficientFunction(tuple(j_impl)) / self.parameters.Ms
  self._j_mp_cf = CoefficientFunction(tuple(j_mp)) / self.parameters.Ms

  self._flag_je_stationary = self.parameters.flag_Je_stationary or flag_je_const

  if not(self._flag_je_stationary):
    jnew = [0.0, 0.0, 0.0]
    for d in range(3):
      if isinstance(self.parameters.Je[d], types.FunctionType):
        jnew[d] = self.parameters.Je[d]((self._time_cf + self._k) / tScale, x, y, z)
      else:
        jnew[d] = CoefficientFunction(self.parameters.Je[d])
  
    jnew_cf = CoefficientFunction(tuple(jnew)) / self.parameters.Ms

    self._jprime_cf = (jnew_cf - self._j_cf) / self._k
  else:
    self._jprime_cf = None
  
  self._j_cf_oersted = self._j_cf * self._geometry.scaling
  self._j_impl_cf_oersted = self._j_impl_cf * self._geometry.scaling
  self._j_mp_cf_oersted = self._j_mp_cf * self._geometry.scaling

  
#------------------------------------------------------------------------------#


def ConvertCurrentDensity_Slonczewski(self):
  import types
  from ngsolve import CoefficientFunction, x, y, z

  flag_je_const = True

  tScale = self.parameters.gamma0 * self.parameters.Ms
  if isinstance(self.parameters.Je, types.FunctionType):
    flag_je_const = False
    self._j_cf = self.parameters.Je(self._time_cf / tScale, x, y, z) \
      / self.parameters.Ms
    self._j_impl_cf = self.parameters.Je( \
      (self._time_cf + self._theta*self._k) / tScale, x, y, z) \
      / self.parameters.Ms
    self._j_mp_cf = self.parameters.Je( \
      (self._time_cf + 0.5*self._k) / tScale, x, y, z) \
      / self.parameters.Ms
  else:
    self._j_cf = CoefficientFunction(self.parameters.Je) / self.parameters.Ms
    self._j_impl_cf = CoefficientFunction(self.parameters.Je) / self.parameters.Ms
    self._j_mp_cf = CoefficientFunction(self.parameters.Je) / self.parameters.Ms
  
  self._flag_je_stationary = self.parameters.flag_Je_stationary or flag_je_const

  if not(self._flag_je_stationary):
    jnew_cf = self.parameters.Je((self._time_cf + self._k) / tScale, x, y, z) \
    / self.parameters.Ms
    self._jprime_cf = (jnew_cf - self._j_cf) / self._k
  else:
    self._jprime_cf = None

  axisMap = {'x': 0, 'y': 1, 'z': 2}
  oopAxis = axisMap[self.parameters.slonczewskiOutOfPlane_axis]
  j_tmp = [0.0, 0.0, 0.0]
  j_impl_tmp = [0.0, 0.0, 0.0]
  j_mp_tmp = [0.0, 0.0, 0.0]
  j_tmp[oopAxis] = self._j_cf * self._geometry.scaling
  j_impl_tmp[oopAxis] = self._j_impl_cf * self._geometry.scaling
  j_mp_tmp[oopAxis] = self._j_mp_cf * self._geometry.scaling

  self._j_cf_oersted = CoefficientFunction(tuple(j_tmp))
  self._j_impl_cf_oersted = CoefficientFunction(tuple(j_impl_tmp))
  self._j_mp_cf_oersted = CoefficientFunction(tuple(j_mp_tmp))

  
#------------------------------------------------------------------------------#


def ConvertPolarizationAxis(self):
  from ngsolve import CoefficientFunction
  self._p_cf = CoefficientFunction(self.parameters.p) #/ p_norm


#------------------------------------------------------------------------------#


################################################################################
